package com.accuver.cem.view.frag;


import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;

import com.accuver.jpcem.R;

import com.samknows.measurement.SK2AppSettings;
import com.samknows.measurement.util.OtherUtils;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnDismissListener;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.text.InputType;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;



public class Frag_Preference  extends Fragment implements OnClickListener{

	private static final String TAG = Frag_Preference.class.getSimpleName();
	private Activity mActivity;
	private SK2AppSettings mSettings;
	
	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		
		mActivity = getActivity();
		mSettings = SK2AppSettings.getSK2AppSettingsInstance();
		View rootView = inflater.inflate(R.layout.frag_preference, container, false);
		findView(rootView);

		return rootView; 
	}
	
	
	private TextView mTitleDataUsed;
	private TextView mTitleMobileCap;
	private TextView mTitleDataResetDay;
	private CheckBox mBGTestCheck;
	private TextView mValueAutoTest;
	private TextView mValueNextTest;
	
	private void findView(View view) {
		view.findViewById(R.id.rlayout_bgtest).setOnClickListener(this);
		view.findViewById(R.id.rlayout_datause).setOnClickListener(this);
		view.findViewById(R.id.rlayout_datacap).setOnClickListener(this);
		view.findViewById(R.id.rlayout_datareset).setOnClickListener(this);
		
		mTitleDataUsed = (TextView)view.findViewById(R.id.tv_title_datause);
		mTitleMobileCap = (TextView)view.findViewById(R.id.tv_title_datacap);
		mTitleDataResetDay = (TextView)view.findViewById(R.id.tv_title_datareset);
		
		mBGTestCheck = (CheckBox)view.findViewById(R.id.cb_bgtest);
		mBGTestCheck.setClickable(false);
		mBGTestCheck.setChecked(mSettings.isServiceEnabled());
		
		mValueAutoTest = (TextView)view.findViewById(R.id.tv_autotest_value);
		mValueNextTest = (TextView)view.findViewById(R.id.tv_nexttest_value);
		
		updateTitle();
	}

	@Override
	public void onClick(View v) {
		switch(v.getId()){
		case R.id.rlayout_bgtest:
			toggleBackgroundTest();
			break;
		case R.id.rlayout_datause:
			resetUsedDataSize();
			break;
		case R.id.rlayout_datacap:
			setDataCapSize();
			break;
		case R.id.rlayout_datareset:
			setUsedDataSizeResetDay();
			break;
		default:
			break;
		}
	}
	
	private void toggleBackgroundTest() {
		boolean isOn = mBGTestCheck.isChecked();
		isOn = !isOn;
		mBGTestCheck.setChecked(isOn);
		mSettings.setServiceEnabled(isOn);
		
		if (isOn) {
			long schedule_time = SK2AppSettings.getInstance().getNextRunTime();
			long current_time = System.currentTimeMillis();
			if(schedule_time < current_time){
				long time = OtherUtils.getNextRunTime();
				OtherUtils.rescheduleWakeup(mActivity, time);			
			}
			else {
				long time = schedule_time - current_time;
				OtherUtils.rescheduleWakeup(mActivity, time);
			}
			//Temp
//			OtherUtils.rescheduleWakeup(mActivity, 2*60*1000);
		}else{
			OtherUtils.cancelAlarm(mActivity);
		}
		
		updateTitle();
	}
	
	private void resetUsedDataSize() {
		AlertDialog.Builder ab = new AlertDialog.Builder(mActivity);
		ab.setTitle(R.string.setting_datause_title);
		ab.setMessage(R.string.setting_question_data_reset);
		ab.setPositiveButton(R.string.setting_alert_yes, new DialogInterface.OnClickListener() {			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				//reset data usage now!
				mSettings.resetDataUsage();
				updateTitle();
			}
		});
		ab.setNegativeButton(R.string.setting_alert_no, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				//nothing
			}
		});
		ab.show();
	}
	
	private EditText mCapEditText;
	private void setDataCapSize() { 
		
		mCapEditText = new EditText(mActivity);
        long mobile_cap = mSettings.getDataCapBytes();
        mobile_cap /= 1024;
        mobile_cap /= 1024;
        mCapEditText.setText(""+mobile_cap);
        mCapEditText.setInputType(InputType.TYPE_CLASS_NUMBER);
        
		AlertDialog.Builder ab = new AlertDialog.Builder(mActivity);
		ab.setTitle(R.string.setting_mobilecap_title);
		ab.setMessage(R.string.setting_mobilecap_detail);
		ab.setView(mCapEditText);
		ab.setPositiveButton(R.string.setting_alert_save, new DialogInterface.OnClickListener() {			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				long mobile_cap = Long.parseLong(mCapEditText.getText().toString());
				//mSettings.setDataCapMbIfNull(mobile_cap);
				mSettings.saveDataCap(mobile_cap);
				updateTitle();
			}
		});
		ab.setNegativeButton(R.string.setting_alert_no, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				//nothing
			}
		});
		AlertDialog dialog = ab.show();
		dialog.setOnDismissListener(new OnDismissListener(){
			@Override
			public void onDismiss(DialogInterface di) {
				Frag_TabMain main = (Frag_TabMain)mActivity;
				main.hideSoftKeyboard(mCapEditText);
				mCapEditText = null;
			}
		});
	}
	
	private void setUsedDataSizeResetDay() {
		LayoutInflater factory = LayoutInflater.from(mActivity);
        final ViewGroup main_view = (ViewGroup)factory.inflate(R.layout.alert_resetday, null);
        
        final Button btnDefault = (Button)main_view.findViewById(R.id.btn_default);
        final TextView tvDay = (TextView)main_view.findViewById(R.id.tv_set_day);
        final SeekBar seekDay = (SeekBar)main_view.findViewById(R.id.seek_day);
        seekDay.setMax(31);
        
        btnDefault.setOnClickListener(new OnClickListener(){
			@Override
			public void onClick(View v) {
				seekDay.setProgress(1);
				tvDay.setText(mActivity.getResources().getString(R.string.setting_day_of_month)+":"+getOrdinal(1));
			}
        });
        seekDay.setOnSeekBarChangeListener(new OnSeekBarChangeListener(){
			@Override
			public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
				if(progress == 0) {
					progress = 1;
					seekBar.setProgress(progress);
				}
				tvDay.setText(mActivity.getResources().getString(R.string.setting_day_of_month)+":"+getOrdinal(progress));
			}
			@Override
			public void onStartTrackingTouch(SeekBar seekBar) {
				
			}
			@Override
			public void onStopTrackingTouch(SeekBar seekBar) {
				
			}
        });
        
        int reset_day = mSettings.getDayResetDataCap();
        tvDay.setText(mActivity.getResources().getString(R.string.setting_day_of_month)+":"+getOrdinal(reset_day));
        seekDay.setProgress(reset_day);
        
		
		AlertDialog.Builder ab = new AlertDialog.Builder(mActivity);
		ab.setTitle(R.string.setting_mobilereset_title);
		ab.setView(main_view);
		ab.setPositiveButton(R.string.setting_alert_save, new DialogInterface.OnClickListener() {			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				int day = seekDay.getProgress();
				mSettings.setDayResetDataCap(day);
				updateTitle();
			}
		});
		ab.setNegativeButton(R.string.setting_alert_no, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				//nothing
			}
		});
		ab.show();
	}
	
	private String getOrdinal(int num){
		if(num < 1) return null;
		
		String diplayLanguage = mActivity.getResources().getConfiguration().locale.getDisplayLanguage();
		//Log.e(TAG,"diplayLanguage:"+diplayLanguage );
		
		String displayCountry = mActivity.getResources().getConfiguration().locale.getDisplayCountry();
		//Log.e(TAG,"DisplayCountry:"+displayCountry );
		
		if(diplayLanguage.equals("") || displayCountry.equals("")){
			return ""+num+"";
		}
		
		String retString;
		if(num == 1) retString = "1st";
		else if(num == 2) retString = "2nd";
		else if(num == 3) retString = "3rd";
		else {
			retString = String.format("%dth",num);
		}
		return retString;
	}
	
	public void updateTitle(){
		if(mSettings == null)return;
		if(mTitleDataUsed == null)return;
		if(mTitleMobileCap == null)return;
		if(mTitleDataResetDay == null)return;
		
		// 1. data used : xxx.xxKB , xxx.xxMB , xxxB
		long data_usage = mSettings.getUsedBytes();
		String sizeString;
		if(data_usage > (1024*1024)){
			sizeString = String.format("%.2fMB", data_usage/1024.0f/1024.0f);
		}
		else if(data_usage > 1024){
			sizeString = String.format("%.2fKB", data_usage/1024.0f);
		}
		else {
			sizeString = String.format("%dB", data_usage);
		}
		mTitleDataUsed.setText(""+mActivity.getResources().getString(R.string.setting_datause_title)+":"+sizeString);
		
		// 2. mobile data cap : 300MB
		long mobile_cap = mSettings.getDataCapBytes();
		sizeString = String.format("%dMB",mobile_cap/1024/1024);
		mTitleMobileCap.setText(""+mActivity.getResources().getString(R.string.setting_mobilecap_title)+":"+sizeString);
		
		// 3. Mobile data usage reset day:1st
		int reset_day = mSettings.getDayResetDataCap();
		sizeString = getOrdinal(reset_day);
		mTitleDataResetDay.setText(""+mActivity.getResources().getString(R.string.setting_mobilereset_title)+":"+sizeString);
		
		// 4. Auto Testing (Enabled or Disabled)
		mValueAutoTest.setText(mSettings.isServiceEnabled() ? R.string.setting_enabled:R.string.setting_disabled);
		
		// 5. Next Test Time
		Calendar calendar = Calendar.getInstance();
		calendar.setTimeInMillis(mSettings.getNextRunTime());
		
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy/MM/dd hh:mm aa",Locale.ENGLISH);
		String timeString = formatter.format(calendar.getTime());
		mValueNextTest.setText(timeString);
		if(mSettings.isServiceEnabled()){
			mValueNextTest.setVisibility(View.VISIBLE);
		}
		else {
			mValueNextTest.setVisibility(View.INVISIBLE);
		}
	}
}
