package com.samknows.measurement;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

import com.samknows.measurement.util.DCSConvertorUtil;

import android.annotation.SuppressLint;
import android.content.Context;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.NetworkInfo.State;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Bundle;
import android.telephony.CellLocation;
import android.telephony.PhoneStateListener;
import android.telephony.SignalStrength;
import android.telephony.TelephonyManager;
import android.telephony.cdma.CdmaCellLocation;
import android.telephony.gsm.GsmCellLocation;
import android.telephony.CellInfo;
import android.util.Log;

public class CollectDeviceInfo {
	
	private final String TAG = CollectDeviceInfo.class.getSimpleName();
	
	private Context context;
	private boolean isUpdatedCellLocation = false;
	private CollectDeviceInfo(Context c) {
		context = c;
		isActive = true;
		new Thread(new Runnable(){
			@SuppressLint("NewApi") 
			@Override
			public void run() {
				long runningTime = 0;
				while(isActive){
					if(runningTime < 1000){
						try {
							Thread.sleep(10000 - runningTime);
						} catch (InterruptedException e) {
							
						}	
					}
					runningTime = System.currentTimeMillis();
					/////////////////////////////////////////////
					ConnectivityManager connectivity = (ConnectivityManager)context.getSystemService(Context.CONNECTIVITY_SERVICE);
					TelephonyManager telephony = (TelephonyManager)context.getSystemService(Context.TELEPHONY_SERVICE);
					// Operator
					operatorName = telephony.getNetworkOperatorName();
					// Network Type
					State wifi_state = connectivity.getNetworkInfo(ConnectivityManager.TYPE_WIFI).getState();
					netType = telephony.getNetworkType();
					if(wifi_state == NetworkInfo.State.CONNECTED){
						net = NET_WIFI;
						WifiManager wifi = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
						WifiInfo wifiInfo = wifi.getConnectionInfo();
						nWIFIRSSI = wifiInfo.getRssi();
					}
					else {
						net = DCSConvertorUtil.convertNetworkType(netType);
					}
					/////////////////////////////////////////////
					runningTime = System.currentTimeMillis() - runningTime;
					
				}
			}
		}).start();
		
		TelephonyManager telephony = (TelephonyManager)context.getSystemService(Context.TELEPHONY_SERVICE);
		telephony.listen(new PhoneStateListener(){
			
			// use hidden api
			private int getSignalStrengthValue(String method_name, String field_name, SignalStrength object){
				// first. try method(over 4.4)
				if(Build.VERSION.SDK_INT >= 19){
					try {
						Integer value = (Integer)SignalStrength.class.getMethod(method_name, (Class[])null).invoke(object, null);
						return (int)value;
					} catch (NoSuchMethodException e) {
						Log.e(TAG,"NoSuchMethodException:"+e.getMessage());
					} catch (IllegalAccessException e) {
						Log.e(TAG,"IllegalAccessException:"+e.getMessage());
					} catch (IllegalArgumentException e) {
						Log.e(TAG,"IllegalArgumentException:"+e.getMessage());
					} catch (InvocationTargetException e) {
						Log.e(TAG,"InvocationTargetException:"+e.getMessage());
					}					
				}
				
				// second. try field(private)
				try {
					Field f = SignalStrength.class.getDeclaredField(field_name);
					f.setAccessible(true);
					int value = f.getInt(object);
					return value;
				} catch (IllegalAccessException e) {
					Log.e(TAG,"IllegalAccessException:"+e.getMessage());
				} catch (IllegalArgumentException e) {
					Log.e(TAG,"IllegalArgumentException:"+e.getMessage());
				} catch (NoSuchFieldException e) {
					Log.e(TAG,"NoSuchFieldException:"+e.getMessage());
					
				}
				
				return -1;
			}
		
			public void onSignalStrengthsChanged (SignalStrength signalStrength){
				if(signalStrength == null) return;
				
				nGSMRSSI = (signalStrength.getGsmSignalStrength() == 99) ? -1 : (-113 + 2*signalStrength.getGsmSignalStrength());
				nCDMARSSI = signalStrength.getCdmaDbm();
				nCDMAECIO = signalStrength.getCdmaEcio();
				nEVDORSSI = signalStrength.getEvdoDbm();
				nEVDOECIO = signalStrength.getEvdoEcio();
				
				//Log.d(TAG,""+signalStrength.toString());
				
				int lteSignalStrength = getSignalStrengthValue("getLteSignalStrength","mLteSignalStrength",signalStrength);
				//Log.d(TAG,"lteSignalStrength:"+lteSignalStrength);
				
				int lteRssi = (lteSignalStrength==99) ? -1 : (-113 + 2*lteSignalStrength);
				int lteRsrp = getSignalStrengthValue("getLteRsrp","mLteRsrp",signalStrength);
				int lteRsrq = (int) (-1*Math.abs((double)getSignalStrengthValue("getLteRsrq","mLteRsrq",signalStrength)));
				float lteRssnr = (float)(0.1*getSignalStrengthValue("getLteRssnr","mLteRssnr",signalStrength));
				
				nLTERSSI = lteRssi;
				nLTERSRP = lteRsrp;
				nLTERSRQ = lteRsrq;
				fLTESNR = lteRssnr;
				
			}
			public void onCellLocationChanged (CellLocation location) {
				//Log.d(TAG,"onCellLocationChanged");
				if(location instanceof GsmCellLocation){
					GsmCellLocation gsm = (GsmCellLocation)location;					
					nLTECELLID = nGSMCELLID = gsm.getCid();
				}
				else if(location instanceof CdmaCellLocation){
					CdmaCellLocation cdma = (CdmaCellLocation)location;
					nCDMABSID = cdma.getBaseStationId();
				}
				else {
					Log.e(TAG,"CellLocation other type!!");
				}
				isUpdatedCellLocation = true;
			}
		}, PhoneStateListener.LISTEN_SIGNAL_STRENGTHS | PhoneStateListener.LISTEN_CELL_LOCATION);

		// GPS
		LocationManager location = (LocationManager)context.getSystemService(Context.LOCATION_SERVICE);
		location.requestLocationUpdates(LocationManager.GPS_PROVIDER, 3000, 3, new LocationListener(){
			@Override
			public void onLocationChanged(Location loc) {
				latitude = loc.getLatitude();
				longitude = loc.getLongitude();
				accuracy = loc.getAccuracy();
				Log.d(TAG,"location latitude:"+latitude+" longitude:"+longitude+" accuracy:"+accuracy);
			}
			@Override
			public void onProviderDisabled(String provider) {}
			@Override
			public void onProviderEnabled(String provider) {}
			@Override
			public void onStatusChanged(String provider, int arg1, Bundle arg2) {}
		});
		
	}
	
	private int nWIFIRSSI;
	private int nGSMRSSI;
	
	private int nLTERSSI;
	private int nLTERSRQ;
	private int nLTERSRP;
	private float fLTESNR;
	
	private int nLTECELLID;	
	private int nGSMCELLID;
	private int nCDMABSID;
	
	private int nCDMARSSI;
	private int nCDMAECIO;
	
	private int nEVDORSSI;
	private int nEVDOECIO;
	
	
	private boolean isActive;
	public void close(){
		isActive = false;
	}

	private static CollectDeviceInfo instance;
	public static final void create(Context c) {
		instance = new CollectDeviceInfo(c);
	}
	
	public static CollectDeviceInfo getInstance() {
		return instance;
	}
	
	private final String NET_WIFI = "WIFI";


	public int getRSSI(){
		if(net != null && net.equals(NET_WIFI)){
			// connected WIFI
			return -1;
		}

		switch(netType){
		case TelephonyManager.NETWORK_TYPE_GPRS:
		case TelephonyManager.NETWORK_TYPE_EDGE: 
		case TelephonyManager.NETWORK_TYPE_UMTS:
		case TelephonyManager.NETWORK_TYPE_HSDPA:
		case TelephonyManager.NETWORK_TYPE_HSUPA:
		case TelephonyManager.NETWORK_TYPE_HSPA:
		case TelephonyManager.NETWORK_TYPE_EHRPD:
		case TelephonyManager.NETWORK_TYPE_HSPAP:
		case TelephonyManager.NETWORK_TYPE_IDEN:
			return nGSMRSSI;
		
		case TelephonyManager.NETWORK_TYPE_CDMA:
		case TelephonyManager.NETWORK_TYPE_1xRTT:
			return nCDMARSSI;
			
		case TelephonyManager.NETWORK_TYPE_EVDO_0:
		case TelephonyManager.NETWORK_TYPE_EVDO_A:
		case TelephonyManager.NETWORK_TYPE_EVDO_B:
			return nEVDORSSI;
			
		case TelephonyManager.NETWORK_TYPE_LTE:
			return nLTERSSI;
		}
		
		return 0;
	}
	
	public int getCellID(){
		if(net != null && net.equals(NET_WIFI)){
			// connected WIFI
			return -1;
		}
		
		if(isUpdatedCellLocation == false){
			Log.e(TAG,"don't receive onLocationChanged");
			TelephonyManager telephony = (TelephonyManager)context.getSystemService(Context.TELEPHONY_SERVICE);
			CellLocation location = telephony.getCellLocation();
			if(location != null){
				if(location instanceof GsmCellLocation){
					GsmCellLocation gsm = (GsmCellLocation)location;					
					nLTECELLID = nGSMCELLID = gsm.getCid();
					Log.d(TAG,"lteCelID:"+nLTECELLID);
				}
				else if(location instanceof CdmaCellLocation){
					CdmaCellLocation cdma = (CdmaCellLocation)location;
					nCDMABSID = cdma.getBaseStationId();
				}
				else {
					Log.e(TAG,"CellLocation other type!!");
				}	
			}
		}
		
		switch(netType){

		case TelephonyManager.NETWORK_TYPE_GPRS:
		case TelephonyManager.NETWORK_TYPE_EDGE: 
		case TelephonyManager.NETWORK_TYPE_UMTS:
		case TelephonyManager.NETWORK_TYPE_HSDPA:
		case TelephonyManager.NETWORK_TYPE_HSUPA:
		case TelephonyManager.NETWORK_TYPE_HSPA:
		case TelephonyManager.NETWORK_TYPE_EHRPD:
		case TelephonyManager.NETWORK_TYPE_HSPAP:
		case TelephonyManager.NETWORK_TYPE_IDEN:
			return nGSMCELLID;
		
		case TelephonyManager.NETWORK_TYPE_CDMA:
		case TelephonyManager.NETWORK_TYPE_1xRTT:
			return nCDMABSID;
			
		case TelephonyManager.NETWORK_TYPE_EVDO_0:
		case TelephonyManager.NETWORK_TYPE_EVDO_A:
		case TelephonyManager.NETWORK_TYPE_EVDO_B:
			return nCDMABSID;
			
		case TelephonyManager.NETWORK_TYPE_LTE:
			return nLTECELLID;
		}
		
		return 0;
	}
	private int netType;
	public String net;
	public String operatorName;
	public double latitude;
	public double longitude;
	public float accuracy;
	public String bestTarget;
	public String bestAddress;
	
}
