package com.samknows.measurement.net;

import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.StatusLine;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.HttpContext;
import org.json.JSONException;
import org.json.JSONObject;

import com.samknows.libcore.SKLogger;
import com.samknows.measurement.CollectDeviceInfo;
import com.samknows.measurement.SK2AppSettings;
import com.samknows.measurement.storage.ExportFile;
import com.samknows.measurement.test.TestResultsManager;

import android.content.Context;
import android.net.Uri;

public class SubmitTestResultsAction {
	protected Context context;
	protected boolean isSuccess = false;

	public static final String START_TIME = "StartTime";
	public static final String END_TIME = "EndTime";
	public static final String TARGET_NAME = "Target_Name";
	public static final String TARGET_IP = "Target_IP"; 
	public static final String REQUESTED_TESTS = "Requested_tests";
	public static final String AVG_DL_THROUGHPUT = "Avg_DL_Throughput";
	public static final String AVG_UL_THROUGHPUT = "Avg_UL_Throughput";
	public static final String MAX_DL_THROUGHPUT = "Max_DL_Throughput";
	public static final String MAX_UL_THROUGHPUT = "Max_UL_Throughput";
	public static final String LATENCY = "Latency";
	public static final String PACKET_LOSS_RATIO = "Packet_Loss_Ratio";
	public static final String GET_RESULT = "GET_Result";
	public static final String GET_THREAD = "GET_Thread";
	public static final String GET_WARMUP_BYTES = "GET_Warmup_Bytes";
	public static final String GET_WARMUP_TIME = "GET_Warmup_Time";
	public static final String GET_BYTE_SEC = "GET_Byte_Sec";
	public static final String GET_TRANSFER_TIME = "GET_Transfer_Time";
	public static final String GET_BYTE_TOTAL = "GET_Byte_Total";
	public static final String POST_RESULT = "POST_Result";
	public static final String POST_THREAD = "POST_Thread";
	public static final String POST_WARMUP_BYTES = "POST_Warmup_Bytes";
	public static final String POST_WARMUP_TIME = "POST_Warmup_Time";
	public static final String POST_BYTE_SEC = "POST_Byte_Sec";
	public static final String POST_TRANSFER_TIME = "POST_Transfer_Time";
	public static final String POST_BYTE_TOTAL = "POST_Byte_Total";
	public static final String RTT_RESULT = "RTT_Result";
	public static final String RTT_AVG = "RTT_AVG";
	public static final String RTT_MIN = "RTT_MIN";
	public static final String RTT_MAX = "RTT_MAX";
	public static final String RTT_STDDEV = "RTT_STDDEV";
	public static final String RECEIVE_PACKETS = "Received_Packets";
	public static final String LOST_PACKET = "Lost_Packet";
	public static final String MODEL = "Model";
	public static final String MANUFACTURER = "Manufacturer";
	public static final String OS_TYPE = "OS_Type";
	public static final String OS_VERSION = "OS_Version";
	public static final String NETWORK_TYPE = "Network_Type";
	public static final String PHONE_TYPE = "Phone_Type";
	public static final String CONNECTED = "Connected";
	public static final String ACTIVE_NETWORK_TYPE = "Active_Network_Type";
	public static final String NETWORK_OPERATOR_NAME = "Network_Operator_Name";
	public static final String NETWORK_OPERATOR_CODE = "Network_Operator_Code";
	public static final String SIM_OPERATOR_NAME = "SIM_Operator_Name";
	public static final String SIM_OPERATOR_CODE = "SIM_Operator_Code";
	public static final String ROAMING = "Roaming";
	public static final String TOTAL_TX_BYTES = "Total_TX_Bytes";
	public static final String MOBILE_TX_BYTES = "Mobile_TX_Bytes";
	public static final String APP_TX_BYTES = "APP_TX_Bytes";
	public static final String TOTAL_RX_BYTES = "Total_RX_Bytes";
	public static final String MOBILE_RX_BYTES = "Mobile_RX_Bytes";
	public static final String APP_RX_BYTES = "APP_RX_Bytes";
	public static final String DURATION = "Duration";
	public static final String LOCATION_TYPE = "Location_Type";
	public static final String LONGITUDE = "Longitude";
	public static final String LATITUDE = "Latitude";
	public static final String ACCURACY = "Accuracy";
	public static final String LTE_RSSI = "LTE_RSSI";
//	public static final String LTE_RSRP = "LTE_RSRP";
//	public static final String LTE_RSRQ = "LTE_RSRQ";
//	public static final String LTE_SINR = "LTE_SINR";
	public static final String LTE_CELLID = "LTE_CellID";
//	public static final String LTE_PCI = "LTE_PCI";
//	public static final String GSM_LAC = "GSM_LAC";
//	public static final String GSM_PSC = "GSM_PSC";
	public static final String GSM_RSSI = "GSM_RSSI";
	public static final String GSM_CELLID = "GSM_CellID";
	public static final String CDMA_RSSI = "CDMA_RSSI";
//	public static final String CDMA_ECIO = "CDMA_ECIO";
	public static final String CDMA_BSID = "CDMA_BSID";
//	public static final String CDMA_NID = "CDMA_NID";
//	public static final String CDMA_SID = "CDMA_SID";
	public static final String EVDO_RSSI = "EVDO_RSSI";
//	public static final String EVDO_ECIO = "EVDO_ECIO";

	public static final String[] TEST_RESULT_FIELDS = {
		START_TIME,
		END_TIME,
		TARGET_NAME,
		TARGET_IP,
		REQUESTED_TESTS,
		AVG_DL_THROUGHPUT,
		AVG_UL_THROUGHPUT,
		MAX_DL_THROUGHPUT,
		MAX_UL_THROUGHPUT,
		LATENCY,
		PACKET_LOSS_RATIO,
		GET_RESULT,
		GET_THREAD,
		GET_WARMUP_BYTES,
		GET_WARMUP_TIME,
		GET_BYTE_SEC,
		GET_TRANSFER_TIME,
		GET_BYTE_TOTAL,
		POST_RESULT,
		POST_THREAD,
		POST_WARMUP_BYTES,
		POST_WARMUP_TIME,
		POST_BYTE_SEC,
		POST_TRANSFER_TIME,
		POST_BYTE_TOTAL,
		RTT_RESULT,
		RTT_AVG,
		RTT_MIN,
		RTT_MAX,
		RTT_STDDEV,
		RECEIVE_PACKETS,
		LOST_PACKET,
		MODEL,
		MANUFACTURER,
		OS_TYPE,
		OS_VERSION,
		NETWORK_TYPE,
		PHONE_TYPE,
		CONNECTED,
		ACTIVE_NETWORK_TYPE,
		NETWORK_OPERATOR_NAME,
		NETWORK_OPERATOR_CODE,
		SIM_OPERATOR_NAME,
		SIM_OPERATOR_CODE,
		ROAMING,
		TOTAL_TX_BYTES,
		MOBILE_TX_BYTES,
		APP_TX_BYTES,
		TOTAL_RX_BYTES,
		MOBILE_RX_BYTES,
		APP_RX_BYTES,
		DURATION,
		LOCATION_TYPE,
		LONGITUDE,
		LATITUDE,
		ACCURACY,
		LTE_RSSI,
//		LTE_RSRP,
//		LTE_RSRQ,
//		LTE_SINR,
		LTE_CELLID,
//		LTE_PCI,
//		GSM_LAC,
//		GSM_PSC,
		GSM_RSSI,
		GSM_CELLID,
		CDMA_RSSI,
//		CDMA_ECIO,
		CDMA_BSID,
//		CDMA_NID,
//		CDMA_SID,
		EVDO_RSSI
//		EVDO_ECIO
	};
	
	
	private String getLongString(JSONObject json, String name){
		String string = "";
		try {
			string = ""+json.getLong(name);
		} catch (JSONException e) {

		}		
		return string;
	}
	private String getString(JSONObject json, String name){
		String string = "";
		try {
			string = ""+json.getString(name);
		} catch (JSONException e) {

		}		
		return string;
	}
	private String getDoubleString(JSONObject json, String name){
		String string = "";
		try {
			string = ""+json.getDouble(name);
		} catch (JSONException e) {

		}		
		return string;
	}
	
	private String startTime = null;
	private String endTime = null;
	private String filename = null;
	private boolean isRepeat;
	private String imsi = null;
	public void setTestInfo(String start, String end, String name, String i, boolean repeat){
		this.startTime = start;
		this.endTime = end;
		this.filename= name;  
		this.isRepeat = repeat;
		this.imsi = i;
	}
	
	public SubmitTestResultsAction(Context _context) {
		context = _context;
	}

	public void execute() {
		String[] results = TestResultsManager.getJSONDataAsStringArray(context);
		List<Integer> fail = new ArrayList<Integer>();
		for (int i=0; i<results.length; i++) {
			String result = results[i];
			if (result == null) {
				SKLogger.d(SubmitTestResultsAction.class,
						"no results to be submitted");
				break;
			}
			
			CollectDeviceInfo device = CollectDeviceInfo.getInstance();
			
			
			try {
				JSONObject json = new JSONObject(result);
				
				String[] rs = new String[TEST_RESULT_FIELDS.length];
				rs[0] = (startTime != null) ? startTime : getLongString(json,START_TIME);
				rs[1] = (endTime != null) ? endTime : getLongString(json,END_TIME);
				rs[2] = device.bestTarget;//getString(json,TARGET_NAME);
				rs[3] = device.bestAddress;//getString(json,TARGET_IP);
				rs[4] = "GET_POST_RTT";//getString(json,REQUESTED_TESTS);
				rs[5] = getDoubleString(json,AVG_DL_THROUGHPUT);
				rs[6] = getDoubleString(json,AVG_UL_THROUGHPUT);
				rs[7] = getDoubleString(json,MAX_DL_THROUGHPUT);
				rs[8] = getDoubleString(json,MAX_UL_THROUGHPUT);
				rs[9] = getDoubleString(json,LATENCY);
				rs[10] = getDoubleString(json,PACKET_LOSS_RATIO);
				rs[11] = getString(json,GET_RESULT);
				rs[12] = getLongString(json,GET_THREAD);
				rs[13] = getLongString(json,GET_WARMUP_BYTES);
				rs[14] = getLongString(json,GET_WARMUP_TIME);
				rs[15] = getLongString(json,GET_BYTE_SEC);
				rs[16] = getLongString(json,GET_TRANSFER_TIME);
				rs[17] = getLongString(json,GET_BYTE_TOTAL);
				rs[18] = getString(json,POST_RESULT);
				rs[19] = getLongString(json,POST_THREAD);
				rs[20] = getLongString(json,POST_WARMUP_BYTES);
				rs[21] = getLongString(json,POST_WARMUP_TIME);
				rs[22] = getLongString(json,POST_BYTE_SEC);
				rs[23] = getLongString(json,POST_TRANSFER_TIME);
				rs[24] = getLongString(json,POST_BYTE_TOTAL);
				rs[25] = getString(json,RTT_RESULT);
				rs[26] = getLongString(json,RTT_AVG);
				rs[27] = getLongString(json,RTT_MIN);
				rs[28] = getLongString(json,RTT_MAX);
				rs[29] = getLongString(json,RTT_STDDEV);
				rs[30] = getLongString(json,RECEIVE_PACKETS);
				rs[31] = getLongString(json,LOST_PACKET);
				rs[32] = getString(json,MODEL);
				rs[33] = getString(json,MANUFACTURER);
				rs[34] = getString(json,OS_TYPE);
				rs[35] = getString(json,OS_VERSION);
				rs[36] = getString(json,NETWORK_TYPE);
				rs[37] = getString(json,PHONE_TYPE);
				rs[38] = getString(json,CONNECTED);
				rs[39] = getString(json,ACTIVE_NETWORK_TYPE);
				rs[40] = getString(json,NETWORK_OPERATOR_NAME);
				rs[41] = getLongString(json,NETWORK_OPERATOR_CODE);
				rs[42] = getString(json,SIM_OPERATOR_NAME);
				rs[43] = getLongString(json,SIM_OPERATOR_CODE);
				rs[44] = getString(json,ROAMING);
				rs[45] = getLongString(json,TOTAL_TX_BYTES);
				rs[46] = getLongString(json,MOBILE_TX_BYTES);
				rs[47] = getLongString(json,APP_TX_BYTES);
				rs[48] = getLongString(json,TOTAL_RX_BYTES);
				rs[49] = getLongString(json,MOBILE_RX_BYTES);
				rs[50] = getLongString(json,APP_RX_BYTES);
				rs[51] = getLongString(json,DURATION);
				rs[52] = getString(json,LOCATION_TYPE);
				rs[53] = getDoubleString(json,LONGITUDE);
				rs[54] = getDoubleString(json,LATITUDE);
				rs[55] = getDoubleString(json,ACCURACY);
				rs[56] = getLongString(json,LTE_RSSI);
//				rs[0] = getLongString(json,LTE_RSRP);
//				rs[0] = getLongString(json,LTE_RSRQ);
//				rs[0] = getDoubleString(json,LTE_SINR);
				rs[57] = getLongString(json,LTE_CELLID);
//				rs[0] = getLongString(json,LTE_PCI);
//				rs[0] = getLongString(json,GSM_LAC);
//				rs[0] = getLongString(json,GSM_PSC);
				rs[58] = getLongString(json,GSM_RSSI);
				rs[59] = getLongString(json,GSM_CELLID);
				rs[60] = getLongString(json,CDMA_RSSI);
//				rs[0] = getLongString(json,CDMA_ECIO);
				rs[61] = getLongString(json,CDMA_BSID);
//				rs[0] = getLongString(json,CDMA_NID);
//				rs[0] = getLongString(json,CDMA_SID);
				rs[62] = getLongString(json,EVDO_RSSI);
//				rs[0] = getLongString(json,EVDO_ECIO);
				
				ExportFile.appendToTSV(filename, TEST_RESULT_FIELDS, rs);
				ExportFile.uploadFTP(imsi, isRepeat, filename);
			} catch (JSONException e) {
				
			}
		}
		
		TestResultsManager.clearResults(context);
		for(int i:fail){
			TestResultsManager.saveResult(context, results[i]);
		}
	}
}
