package com.samknows.measurement.schedule;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.util.Log;

import com.samknows.measurement.schedule.datacollection.BaseDataCollector;
import com.samknows.measurement.schedule.datacollection.EnvironmentDataCollector;
import com.samknows.measurement.schedule.datacollection.LocationDataCollector;
import com.samknows.tests.Param;
import com.samknows.tests.TestFactory;



public class ScheduleConfig implements Serializable {
	
	private static final String TAG = ScheduleConfig.class.getSimpleName();
	
	
	
	//strings found in the schedule config
	public static final String CONFIG 				= "config";
	public static final String GLOBAL 				= "global";
	public static final String BACKGROUND_TEST		= "background-test";
	public static final String SCHEDULE_VERSION 	= "schedule-version";
	public static final String SUBMIT_DCS 			= "submit-dcs";
	public static final String TESTS_ALARM_TYPE 	= "tests-alarm-type";
	public static final String LOCATION_SERVICE 	= "location-service";
	public static final String ONFAIL_TEST_ACTION	= "onfail-test-action";
	public static final String INIT 				= "init";
	public static final String TYPE 				= "type";
	public static final String HOSTS 				= "hosts";
	public static final String HOST 				= "host";
	public static final String DNSNAME 				= "dnsName";
	public static final String DISPLAYNAME			= "displayName";
	public static final String DATA_CAP_DEFAULT 	= "data-cap-default";
	public static final String VALUE 				= "value";
	public static final String COMMUNICATIONS 		= "communications";
	public static final String COMMUNICATION 		= "communication";
	public static final String DATA_COLLECTOR 		= "data-collector";
	public static final String TIME 				= "time";
	public static final String RANDOM_INTERVAL 		= "random-interval";
	public static final String LISTENERDELAY 		= "listenerDelay";
	public static final String ENABLED 				= "enabled";
	public static final String DISABLED				= "disabled";
	public static final String CONDITIONS 			= "conditions";
	public static final String CONDITION 			= "condition";
	public static final String CONDITION_GROUP 		= "condition-group";
	public static final String CONDITION_GROUP_ID 	= "condition-group-id";
	public static final String FAIL_QUIET			= "fail-quiet";
	public static final String ID 					= "id";
	public static final String TESTS 				= "tests";
	public static final String TEST 				= "test";
	public static final String SCHEDULED_TESTS 		= "scheduled-tests";
	public static final String CONTINUOUS_TESTS		= "continuous-tests";
	public static final String BATCH 				= "batch";
	public static final String MANUAL_TESTS 		= "manual-tests";
	
	private static final long serialVersionUID = 1L;
	
	public boolean backgroundTest = true;
	public List<TestDescription> tests = new ArrayList<TestDescription>(); //scenario.json
	public List<ServerInfo> hosts = new ArrayList<ServerInfo>(); //testServer.json
	public List<BaseDataCollector> dataCollectors = new ArrayList<BaseDataCollector>();
	
	public enum LocationType {
		gps, network
	}
	
	public String findHostName(String name) {
		if(name == null) return null;
		
		for(ServerInfo info : hosts){
			if(info.name.equalsIgnoreCase(name)){
				return info.address;
			}
		}
		return name;
	}
	
	public String findHostByAddress(String address) {
		if(address == null) return null;
		
		for(ServerInfo info : hosts){
			if(info.address.equalsIgnoreCase(address)){
				return info.name;
			}
		}
		return address;
	}
	
	public static ScheduleConfig parseJSON(JSONArray servers, JSONObject scenario){
		
		ScheduleConfig c = new ScheduleConfig();
		
		// add servers
		for(int i=0; i<servers.length(); i++){
			try {
				c.hosts.add(ServerInfo.parseJSON(servers.getJSONObject(i)));
			} catch (JSONException e) {
				
			}
		}
		
		// add scenario
		JSONArray names = scenario.names();
		for(int i=0; i<names.length(); i++) {
			try {
				String name = names.getString(i);
				JSONObject value = scenario.getJSONObject(name);
				TestDescription test = TestDescription.parseJSON(name, value);
				c.tests.add(test);
			} catch (JSONException e) {
				
			}
		}
		
		// add collector
		EnvironmentDataCollector environment = new EnvironmentDataCollector();
		LocationDataCollector location = new LocationDataCollector();
		environment.isEnabled = true;
		location.isEnabled = true;
		c.dataCollectors.add(environment);
		c.dataCollectors.add(location);
		
		return c;
	}

	public TestDescription makeClosestTarget() {
		
		TestDescription td = new TestDescription();
		td.type = TestFactory.CLOSESTTARGET;
		td.displayName = "ClosestTarget";
		td.params = new ArrayList<Param>();
		
		// add all server address
		for(ServerInfo info : hosts){
			if(info == null) continue;
			td.params.add(new Param(TestFactory.TARGET, info.address));
			td.params.add(new Param("port", info.latencyPort));
		}
		
		// set Latency setting
		for(TestDescription one : tests){
			if(one.displayName.equalsIgnoreCase("latency")){
				for(Param param : one.params){
					String name = param.getName();
					if(name.equals(TestFactory.TARGET)){
						continue;
					}
					if(name.equals(TestFactory.NUMBEROFPACKETS)){
						td.params.add(new Param(TestFactory.NUMBEROFPACKETS,"10"));
						continue;
					}
					if(name.equals(TestFactory.MAXTIME)){
						td.params.add(new Param(TestFactory.MAXTIME,"5000000"));
						continue;
					}
					td.params.add(param);
				}
				break;
			}
		}

		return td;
	}

	public void updateTarget(String string) {
		Log.d(TAG,"update "+string);
		String[] array = string.split(";");
		String address = array[4];
		
		for(TestDescription td : tests){
			if(td.type != null && td.type.length() > 0){
				// delete all target
				for(int i=0; i<td.params.size(); i++){
					for(Param param : td.params){
						String name = param.getName();
						if(name.equals(TestFactory.TARGET)){
							td.params.remove(param);
							break;
						}
						if(name.equals(TestFactory.PORT)){
							td.params.remove(param);
							break;
						}
					}	
				}
				
				// add target
				td.params.add(new Param(TestFactory.TARGET,address));
				if(td.type.equalsIgnoreCase(TestFactory.LATENCY)){
					td.params.add(new Param(TestFactory.PORT,"8001"));
				}
				else {
					td.params.add(new Param(TestFactory.PORT,"8000"));
				}
			}
		}
	}
		
}
