package com.samknows.measurement.statemachine.state;

import com.samknows.libcore.SKLogger;
import com.samknows.measurement.SK2AppSettings;
import com.samknows.measurement.CachingStorage;
import com.samknows.measurement.MainService;
import com.samknows.measurement.Storage;
import com.samknows.measurement.net.RequestScheduleAnonymousAction;
import com.samknows.measurement.schedule.ScheduleConfig;
import com.samknows.measurement.statemachine.StateResponseCode;

public class DownloadConfigAnonymousState extends BaseState {

	public DownloadConfigAnonymousState(MainService c) {
		super(c);
	}

	@Override
	public StateResponseCode executeState() throws Exception {
		ScheduleConfig config = null;

		Storage storage = CachingStorage.getInstance();
		SK2AppSettings appSettings  = SK2AppSettings.getSK2AppSettingsInstance();
		RequestScheduleAnonymousAction action = new RequestScheduleAnonymousAction(
				ctx);
		action.execute();
		if (action.isSuccess() == false) {
			SKLogger.sAssert(this.getClass(), "schedule parsing", false);
		} else {
			
			try {
				//config = ScheduleConfig.parseXml(action.content);
				config = ScheduleConfig.parseJSON(action.testServer, action.testInfo);
			} catch (Exception e) {
				// Catch, and rethrow, the exception - as we'd seen this fail in some circumstances and needed to track it down.
		        SKLogger.sAssert(this.getClass(), "error parsing JSON!", false);
				throw(e);
			}

			if(config == null){
				return StateResponseCode.FAIL;
			}
			SKLogger.d(this, "obtained config from server");
			
			appSettings.onlyDownload = false;
			if (appSettings.updateConfig(config)) {
				storage.saveScheduleConfig(config);
				SKLogger.d(this, "Update config");
			} else {
				SKLogger.d(this, "Update config is not necessary");
				return StateResponseCode.OK;
			}
			storage.dropExecutionQueue();
			storage.dropParamsManager();
			appSettings.setConfig(config);
			return StateResponseCode.NOT_OK;
		}

		return StateResponseCode.FAIL;
	}
}
