package com.samknows.measurement.statemachine.state;



import android.content.Context;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import com.samknows.measurement.ManualTest;
import com.samknows.measurement.SK2AppSettings;
import com.samknows.measurement.MainService;
import com.samknows.measurement.SKApplication;
import com.samknows.measurement.statemachine.StateResponseCode;
import com.samknows.measurement.util.WaitQueue;

public class ExecuteScheduledTestQueueState extends BaseState{
	private static final String TAG = ExecuteScheduledTestQueueState.class.getSimpleName();

	private Context mContext;
	public ExecuteScheduledTestQueueState(MainService ctx) {
		super(ctx);
		mContext = ctx;
		mHandler = new Handler(ctx.getMainLooper()){
			@Override
			public void handleMessage(Message msg) {
				if(msg.what == WaitQueue.MSG_WAITQUEUE_EMPTY){
					completeWaitQueue = true;
					Log.d(TAG,"receive waitqueue complete!");
					return;
				}
				else if(msg.what == WaitQueue.MSG_WAITQUEUE_WAIT){
					int count = msg.arg1;
					Log.d(TAG,"receive waitqueue count:"+count);
					return;
				}
			}
		};
	}

	private boolean completeWaitQueue = false;
	private Handler mHandler;
	
	private ManualTest mt;
	private WaitQueue mWaitQueue;
	private void tryTest(){
		if(ManualTest.isExecuting){
			mt = null;
			return;
		}
		
		StringBuilder errorDescription = new StringBuilder();
		// create a new thread
		mt = ManualTest.create(ctx, mHandler, errorDescription);
		if (mt == null)  return;
		mt.setIsRepeatTest(false);
		
		String target = mt.getTargetAddress();
		int port = mt.getSpeedTestPost();
		WaitQueue queue = new WaitQueue(target,port);
		queue.check(mHandler);
		mWaitQueue = queue;
	}
	
	@Override
	public StateResponseCode executeState() {
		if(!SK2AppSettings.getInstance().isServiceEnabled()){
			return StateResponseCode.OK;
		}
		
//		Storage storage = CachingStorage.getInstance();
//		TestContext tc = TestContext.createBackgroundTestContext(ctx);
		
//		ScheduledTestExecutionQueue queue = storage.loadQueue();
//		if (queue == null) {
//			Log.w(getClass().getName(), "fail to load execution queue, creating new...");
//			queue = new ScheduledTestExecutionQueue(tc);
//		} else {
//			queue.setTestContext(tc);
//		}
//		
//		long testRun = queue.executeReturnRescheduleDurationMilliseconds();
//		
//		storage.saveExecutionQueue(queue);
//		storage.saveTestParamsManager(tc.paramsManager);
//		
//		// schedule from Queue or from config refresh
//		OtherUtils.reschedule(ctx, testRun);
		
		
		tryTest();
		if (mt == null || ManualTest.isExecuting) {
			return StateResponseCode.NOT_OK;
		}
		SKApplication.getAppInstance().setManualTest(mt);
		notiActivity();
		while(!mWaitQueue.isSuccess()){
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				break;
			}
		}
		
		Thread testThread = new Thread(mt,"BGTEST_ManualTest");
		testThread.start();
		try {
			testThread.join();
		} catch (InterruptedException e) {

		}
		SKApplication.getAppInstance().setManualTest(null);
		return StateResponseCode.OK;
	}
	
	private void notiActivity(){
		Handler handler = SKApplication.getAppInstance().getActivityHandler();
		if(handler != null){
			handler.sendMessage(handler.obtainMessage(SKApplication.MSG_BGTEST_START));	
		}
	}
}
