package com.samknows.measurement.statemachine.state;

import android.util.Log;

import com.samknows.libcore.SKLogger;
import com.samknows.measurement.CachingStorage;
import com.samknows.measurement.MainService;
import com.samknows.measurement.Storage;
import com.samknows.measurement.schedule.ScheduleConfig;
import com.samknows.measurement.schedule.TestDescription;
import com.samknows.measurement.statemachine.StateResponseCode;
import com.samknows.measurement.test.TestContext;
import com.samknows.measurement.test.TestExecutor;
import com.samknows.measurement.test.TestResult;

public class RunInitTestsState extends BaseState{
	
	private static final String TAG = RunInitTestsState.class.getSimpleName();

	public RunInitTestsState(MainService c) {
		super(c);
	}

	@Override
	public StateResponseCode executeState() {
		Storage storage = CachingStorage.getInstance();
		ScheduleConfig sc = storage.loadScheduleConfig();
		if(sc  == null){
			SKLogger.e(this, "There is no schedule config");
			return StateResponseCode.FAIL;
		}
		TestContext testContext = TestContext.createBackgroundTestContext(ctx);
		TestExecutor initTestExecutor = new TestExecutor(testContext);
		initTestExecutor.startInBackGround();
		
		TestDescription td = sc.makeClosestTarget();
		TestResult result = initTestExecutor.execute(td);
		if (!result.isSuccess) {
			Log.e(TAG,"Fail RunInitTestsState(ClosestTarget Test)");
			return StateResponseCode.FAIL;
		}
		initTestExecutor.stop();
		//initTestExecutor.save("init_test");
		sc.updateTarget(result.results.get(0));
		storage.saveScheduleConfig(sc);

		
		return StateResponseCode.OK;
	}

}
