package com.samknows.measurement.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.CookieHandler;
import java.net.CookieManager;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

import org.json.JSONException;
import org.json.JSONObject;

import android.os.Handler;
import android.os.Message;
import android.util.Log;


public class WaitQueue {
	
	private static final String TAG = WaitQueue.class.getSimpleName();
	
	/*
	 * MSG 9990 ~ 9999
	 */
	public static final int MSG_WAITQUEUE_EMPTY 		= 9999;
	public static final int MSG_WAITQUEUE_WAIT			= 9998;
	public static final int MSG_WAITQUEUE_FAIL			= 9997;

	
	
	private String mTarget;
	private int mPort;
	
	public WaitQueue(String target, int port) {
		mTarget = target;
		mPort = port;
		
		CookieManager cookieManager = new CookieManager();
		CookieHandler.setDefault(cookieManager);
	}
	
	private Handler mHandler;
	public void check(Handler handler) {
		mHandler = handler;
		
		new Thread(new Runnable(){

			@Override
			public void run() {
				isActive = true;
				send(null);
			}
			
		}).start();
	}
	
	public boolean isSuccess(){
		return isSuccess;
	}
	
	private boolean isSuccess;
	private boolean isActive;
	private void send(String Cookie)
	{
		String urlString = "http://"+mTarget+":"+mPort+"/"+"q";
		
		while(true)
		{
			try {
				URL myURL = new URL(urlString);
				URLConnection conn = myURL.openConnection();
				
				if(Cookie != null)
				{
					conn.setRequestProperty("Cookie", Cookie);
					conn.connect();

					
//					Iterator<String> keys = conn.getHeaderFields().keySet().iterator();
//					while(keys.hasNext()){
//						String key = keys.next();
//						if(key != null){
//							String value = conn.getHeaderField(key);
//							Log.d(TAG,""+key+":"+value);
//							if(key.equalsIgnoreCase("Set-Cookie")){
//								cookie = value;
//								cookie = cookie.substring(0, cookie.indexOf(";")+1);
//							}
//						}
//					}
					Log.d(TAG,"cookie:"+Cookie);
					
					InputStream is = conn.getInputStream();
					ByteArrayOutputStream byte_os = new ByteArrayOutputStream(512);
					byte buffer[] = new byte[512];
					
					for(int readByte =0; (readByte = is.read(buffer, 0, buffer.length)) >0;){
						byte_os.write(buffer, 0, readByte);
					}
					
					String readString = new String(byte_os.toByteArray());
					JSONObject json = new JSONObject(readString);
					Log.e(TAG,"read:"+readString);
					
					int q_number = json.getInt("q");
					if(q_number == 0){
						if(mHandler != null){
							Message msg = Message.obtain(mHandler, MSG_WAITQUEUE_EMPTY);
							mHandler.sendMessage(msg);
						}
						isActive = false;
						isSuccess = true;
						return;
					}
					else if(q_number > 0 && Cookie != null){
						Message msg = Message.obtain(mHandler, MSG_WAITQUEUE_WAIT, q_number, 0);
						mHandler.sendMessage(msg);
						Thread.sleep(1000);
						if(isActive)continue;
					}
				}
				else {
					isSuccess = false;
					String headerName = null;
					for (int i=1; (headerName = conn.getHeaderFieldKey(i))!=null; i++) {
					 	if (headerName.equals("Set-Cookie")) {                  
					 		String cookie = conn.getHeaderField(i);
					 		Cookie = cookie;
					 		break;
					 	}
					}
					if(Cookie != null) continue;
				}
				break;
				
			} catch (MalformedURLException e) {
				Log.e(TAG,""+e.getMessage());
				break;
			} catch (IOException e) {
				Log.e(TAG,""+e.getMessage());
				break;
			} catch (JSONException e) {
				Log.e(TAG,""+e.getMessage());
				break;
			} catch (InterruptedException e) {
				Log.e(TAG,""+e.getMessage());
				break;
			}
		}
				
		//FAIL
		{
			Message msg = Message.obtain(mHandler, MSG_WAITQUEUE_FAIL);
			mHandler.sendMessage(msg);
		}
	}


	public void cancel() {
		isActive = false;
	}
	

}
