package com.samknows.ska.activity;

import org.json.JSONException;
import org.json.JSONObject;

import android.annotation.TargetApi;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.samknows.libcore.R;
import com.samknows.libcore.SKLogger;
import com.samknows.measurement.MainService;
import com.samknows.measurement.activity.BaseLogoutActivity;
import com.samknows.measurement.activity.components.UIUpdate;
import com.samknows.measurement.activity.components.Util;

public class SKAActivationActivity extends BaseLogoutActivity {

	private static final String TAG = SKAActivationActivity.class.getSimpleName();
	public Handler handler;
	
	private Dialog mFailDialog = null;
	private boolean isActivationRun = false;

	@TargetApi(Build.VERSION_CODES.HONEYCOMB) 
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE); 
		setFinishOnTouchOutside(false);
		setContentView(R.layout.ska_activation_activity);
		
		Util.initializeFonts(this);
		Util.overrideFonts(this, findViewById(android.R.id.content));

		/*
		 * {"type":"mainprogress", "value":"42"} {"type":"activating"}
		 * {"type":"download"} {"type":"inittests", "total":"24",
		 * "finished":"21", "currentbest":"london", "besttime": "25 ms"}
		 * {"type":"completed"}
		 * {"total":2,"besttime":"-","finished":0,"type":"inittests","currentbest":"-"}
		 */

		handler = new Handler() {
			@Override
			public void handleMessage(Message msg) {
				TextView tv;

				JSONObject message_json;
				if (msg.obj == null) {
					return;
				}
				message_json = (JSONObject) msg.obj;

				try {
					String type = message_json.getString(UIUpdate.JSON_TYPE);

					if (type == UIUpdate.JSON_MAINPROGRESS) {
						String value = message_json
								.getString(UIUpdate.JSON_VALUE);
						ProgressBar progressbar = (ProgressBar) findViewById(R.id.activation_progress);
						int progress = Integer.parseInt(value);
						progressbar.setProgress(progress);
					} else if (type == UIUpdate.JSON_ACTIVATED) {
						ProgressBar pb = (ProgressBar) findViewById(R.id.activating_progress);
						pb.setVisibility(View.GONE);
						ImageView iv = (ImageView) findViewById(R.id.activating_complete);
						iv.setVisibility(View.VISIBLE);
					} else if (type == UIUpdate.JSON_DOWNLOADED) {
						ProgressBar pb = (ProgressBar) findViewById(R.id.download_progress);
						pb.setVisibility(View.GONE);
						ImageView iv = (ImageView) findViewById(R.id.download_complete);
						iv.setVisibility(View.VISIBLE);
					} else if (type == UIUpdate.JSON_INITTESTS) {
						String total = message_json.getString(UIUpdate.JSON_TOTAL);
						String finished = message_json.getString(UIUpdate.JSON_FINISHED);
						String currentbest = message_json.getString(UIUpdate.JSON_CURRENTBEST);
						String besttime = message_json.getString(UIUpdate.JSON_BESTTIME);
						tv = (TextView) findViewById(R.id.currentbest);
						tv.setText(currentbest);
						tv = (TextView) findViewById(R.id.besttime);
						tv.setText(besttime);
						tv = (TextView) findViewById(R.id.server_status);
						tv.setText(finished + " " + getString(R.string.of)
								+ " " + total);

					} else if (type == UIUpdate.JSON_COMPLETED) {
						isActivationRun = false;
						if(mFailDialog != null){
							Log.e(TAG,"receive JSON_COMPLETED"); return;
						}

						ProgressBar progressbar = (ProgressBar) findViewById(R.id.activation_progress);
						progressbar.setProgress(100);
						ProgressBar pb = (ProgressBar) findViewById(R.id.activating_progress);
						pb.setVisibility(View.GONE);
						ImageView iv = (ImageView) findViewById(R.id.activating_complete);
						iv.setVisibility(View.VISIBLE);
						handler.postDelayed(new Runnable(){
							@Override
							public void run() {
								SKAActivationActivity.this.setResult(Activity.RESULT_OK);
								SKAActivationActivity.this.finish();
							}
						}, 1000);
					} else if (type == UIUpdate.JSON_FAILED) {
						// ??
						mFailDialog = new AlertDialog.Builder(SKAActivationActivity.this)
						.setTitle("Information")
						.setMessage(R.string.not_detected_neighbor_test_server)
						.setCancelable(false)
						.setPositiveButton(R.string.find_test_server_alert_retry, new DialogInterface.OnClickListener() {
							public void onClick(DialogInterface dialog, int id) {
								if (MainService.registerActivationHandler(SKAActivationActivity.this, handler) && !isActivationRun) {
									isActivationRun = true;
									mFailDialog = null;
									SKLogger.d(this, "activation handler registered");
								} 
							}
						})
						.setNegativeButton(R.string.find_test_server_alert_cancel, new DialogInterface.OnClickListener() {
							@Override
							public void onClick(DialogInterface dialog, int which) {
								SKAActivationActivity.this.setResult(Activity.RESULT_CANCELED);
								SKAActivationActivity.this.finish();
							}
						}).show();
					}
					else {
						Log.e(TAG,"UIUpdate.JSON_TYPE:"+type);
					}

				} catch (JSONException e) {
					SKLogger.e(SKAActivationActivity.class,
							"Error in parsing JSONObject: " + e.getMessage());

				}
			}
		};
		if (MainService.registerActivationHandler(this, handler)) {
			isActivationRun = true;
			SKLogger.d(this, "activation handler registered");
		} 
	}
	
	@Override
	public void onResume() {
		super.onResume();
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		MainService.unregisterActivationHandler();
	}
	
	@Override
	public void onBackPressed() {
		/*
		 * Nothing
		 * Block back key action
		 */
	}
}
