package com.samknows.tests;

//import android.annotation.SuppressLint;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.atomic.AtomicBoolean;

import org.apache.commons.net.ntp.TimeStamp;

import android.util.Log;

import com.samknows.libcore.SKLogger;
import com.samknows.measurement.CachingStorage;
import com.samknows.measurement.CollectDeviceInfo;
import com.samknows.measurement.Storage;
import com.samknows.measurement.schedule.ScheduleConfig;
import com.samknows.measurement.schedule.TestDescription;
import com.samknows.measurement.storage.ExportFile;
import com.samknows.measurement.util.CustomTimestamp;
import com.samknows.measurement.util.SKDateFormat;
import com.samknows.tests.HttpSingleTest.StateListener;

public class HttpTest extends Test {

	
	private static final String TAG = HttpTest.class.getSimpleName();

	/*
	 * error codes and contraints
	 */
	private static final int BYTESREADERR = -1;
	public static final int MAXNTHREADS = 100;

	/*
	 * Messages regarding the status of the test
	 */
	private static final String HTTPGETRUN = "Running download test";
	private static final String HTTPGETDONE = "Download test completed";
	private static final String HTTPPOSTRUN = "Running upload test";
	private static final String HTTPPOSTDONE = "Upload completed";

	/*
	 * Test strings
	 */
	public static final String DOWNSTREAMSINGLE = "JHTTPGET";
	public static final String DOWNSTREAMMULTI = "JHTTPGETMT";
	public static final String UPSTREAMSINGLE = "JHTTPPOST";
	public static final String UPSTREAMMULTI = "JHTTPPOSTMT";
	/*
	 * Parameters name for the setParameter function
	 */
	private static final String DOWNSTREAM = "downstream";
	private static final String UPSTREAM = "upstream";

	public static final String JSON_TRANFERTIME = "transfer_time";
	public static final String JSON_TRANFERBYTES = "transfer_bytes";
	public static final String JSON_BYTES_SEC = "bytes_sec";
	public static final String JSON_WARMUPTIME = "warmup_time";
	public static final String JSON_WARMUPBYTES = "warmup_bytes";
	public static final String JSON_NUMBER_OF_THREADS = "number_of_threads";
	public static final String JSON_PEAK_BYTES_SEC = "peak_bytes_sec";
	public static final String JSON_BYTES_SEC_AFTER_1SEC = "bytes_sec_after_1sec";

	/*
	 * Time in microseconds
	 */
	private static long microTime() {
		return System.nanoTime() / 1000;
	}

	public HttpTest() {
	}

	public HttpTest(boolean downstream) {
		this.downstream = downstream;
	}

	public HttpTest(String direction) {
		setDirection(direction);
	}

	public String getStringID() {
		String ret = "";
		if (downstream) {
			if (nThreads == 1) {
				ret = DOWNSTREAMSINGLE;
			} else {
				ret = DOWNSTREAMMULTI;
			}
		} else {
			if (nThreads == 1) {
				ret = UPSTREAMSINGLE;
			} else {
				ret = UPSTREAMMULTI;
			}
		}
		return ret;
	}
	
	/*
	 * 
	 */
	private boolean isInvidualTest = true;
	private HttpSingleTest[] invidualTests = null;
	

	/*
	 * 
	 */
	@Override
	public int getNetUsage() {
		return transferBytes + warmupBytes;
	}

	// @SuppressLint("NewApi")
	@Override
	public boolean isReady() {
		if (target.length() == 0) {
			setError("Target empty");
			return false;
		}
		if (port == 0) {
			setError("Port is zero");
			return false;
		}
		if (warmupMaxTime == 0 && warmupMaxBytes == 0) {
			setError("No warmup parameter defined");
			return false;
		}
		if (transferMaxTime == 0 && transferMaxBytes == 0) {
			setError("No transfer parameter defined");
			return false;
		}
		if (!downstream && (sendDataChunkSize == 0 || postDataLength == 0)) {
			setError("Upload parameter missing");
			return false;
		}
		if (downstream && bufferSize == 0) {
			setError("Buffer size missing for download");
			return false;
		}
		if (nThreads < 1 && nThreads > MAXNTHREADS) {
			setError("Number of threads error, current is: " + nThreads
					+ ". Min " + 1 + " Max " + MAXNTHREADS + ".");
			return false;
		}
		return true;
	}

	@Override
	public boolean isSuccessful() {
		return testStatus.equals("OK");
	}


	public int getSendBufferSize() {
		return sendBufferSize;
	}

	public int getReceiveBufferSize() {
		return receiveBufferSize;
	}

	private int nCountActiveThread;
	private boolean isActiveTest;
	@Override
	public void execute() {
		isActiveTest = true;
		/////////////////////////////////////////////
		// Active Logging thread
		if(loggingFileName != null){
			new Thread(new Runnable(){
				@Override
				public void run() {
					long runningTime = 0;
					int transferOffsetBytes = 0;
					int warmupOffsetBytes = 0;
					int transferBytePer500 = 0;
					int warmupBytePer500 = 0;
					
					CollectDeviceInfo device = CollectDeviceInfo.getInstance();
					
					currentSpeed = 0;
					peakSpeed = 0;
					while(isActiveTest){
						if(runningTime < 500){
							try {
								Thread.sleep(500 - runningTime);
								runningTime = System.currentTimeMillis();
							} catch (InterruptedException e) {

							}	
						}

						// warmup byte 
						warmupBytePer500 = warmupBytes - warmupOffsetBytes;
						warmupOffsetBytes = warmupBytes;

						
						// transfer byte
						transferBytePer500 = transferBytesAfter1Sec - transferOffsetBytes;
						if(transferBytePer500 > 0){
							currentSpeed = getSpeed(transferBytePer500, 500000);
							peakSpeed = (currentSpeed > peakSpeed) ? currentSpeed : peakSpeed;	
						}
						else {
							currentSpeed = 0;
						}
						transferOffsetBytes = transferBytesAfter1Sec;
						
						// save data
						String[] data = new String[ExportFile.TEST_RESULT_ENGINEER_FIELDS.length];
						data[0] = new CustomTimestamp(System.currentTimeMillis()){}.toString();	//Time
						data[1] = downstream ? ""+nCountActiveThread : "0";			// GET_thread
						data[2] = downstream ? ""+warmupBytePer500 : "0";			// GET_warmup_Bytes
						data[3] = downstream ? ""+transferBytePer500 : "0";			// GET_Bytes
						data[4] = !downstream ? ""+nCountActiveThread : "0";		// POST_thread
						data[5] = !downstream ? ""+warmupBytePer500 : "0";			// POST_warmup_Bytes
						data[6] = !downstream ? ""+transferBytePer500 : "0";		// POST_Bytes
						data[7] = device.net;																	// network type
						data[8] = device.net;																	// active network type
						data[9] = device.getRSSI() == -1 ? "" : ""+device.getRSSI();																// RSSI
						ExportFile.appendToCSV(loggingFileName, ExportFile.TEST_RESULT_ENGINEER_FIELDS, data);
						
						runningTime = System.currentTimeMillis() - runningTime;
					}
				}
			},"HTTPTest 500ms").start();	
		}
		/////////////////////////////////////////////
		// Session Count
		Storage storage = CachingStorage.getInstance();
        ScheduleConfig config = storage.loadScheduleConfig();
        for(TestDescription td : config.tests){
        	if(td.type != null && td.type.equalsIgnoreCase(TestFactory.DOWNSTREAMTHROUGHPUT)){
        		for(Param param : td.params){
        			String name = param.getName();
        			String value = param.getValue();
        			if(name.equalsIgnoreCase("Session")){
        				nSessionCountDownload = Integer.parseInt(value);
        				break;
        			}
        		}
        	}
        	else if(td.type != null && td.type.equalsIgnoreCase(TestFactory.UPSTREAMTHROUGHPUT)){
        		for(Param param : td.params){
        			String name = param.getName();
        			String value = param.getValue();
        			if(name.equalsIgnoreCase("Session")){
        				nSessionCountUpload = Integer.parseInt(value);
        				break;
        			}
        		}
        	}
        }
		/////////////////////////////////////////////
		Log.d(TAG,"warmup maxTime:"+warmupMaxTime+" maxByte:"+warmupMaxBytes);
		Log.d(TAG,"tranfer maxTime:"+transferMaxTime+" maxByte:"+transferMaxBytes);
		
		if (downstream) {
			infoString = HTTPGETRUN;
			nThreads = nSessionCountDownload;
		} else {
			infoString = HTTPPOSTRUN;
			nThreads = nSessionCountUpload;
		}
		/////////////////////////////////////////////
		
		start();
		final Thread[] threads = new Thread[nThreads];
		if(isInvidualTest)
		{
			invidualTests = new HttpSingleTest[nThreads];
			for (int i = 0; i < nThreads; i++) {
				invidualTests[i] = new HttpSingleTest(downstream);				
				invidualTests[i].setTarget(target);
				invidualTests[i].setPort(port);
				invidualTests[i].setWarmupMaxTime((int)warmupMaxTime);
				invidualTests[i].setWarmupMaxBytes(warmupMaxBytes);
				invidualTests[i].setTransferMaxTime((int)transferMaxTime);
				invidualTests[i].setTransferMaxBytes(transferMaxBytes);
				if(downstream){
					invidualTests[i].setBufferSize(bufferSize);
				}
				else {
					invidualTests[i].setSendDataChunk(sendDataChunkSize);
					invidualTests[i].setPostDataLenght(postDataLength);
				}
				invidualTests[i].setStateListener(new StateListener(){
					
					private boolean isFinishAllWarmup(){
						int count = 0;
						for (int i = 0; i < nThreads; i++){
							if(invidualTests[i].warmUpDone){
								count++;
							}
						}
						warmupDoneCounter = count;
						return (warmupDoneCounter == nThreads);
					}
					
					private long getFastStartWarmup(){
						long start = 0;
						for (int i = 0; i < nThreads; i++) {
							if(start == 0) start = invidualTests[i].getStartWarmup();
							else start = (invidualTests[i].getStartWarmup() < start) ?  invidualTests[i].getStartWarmup() : start;
						}
						return start;
					}
					
					@Override
					synchronized public void noti(HttpSingleTest test) {
						// warmup Byte , warmup time
						if(!warmUpDone) {
							int bytes = 0;
							for (int i = 0; i < nThreads; i++) {
								bytes += invidualTests[i].getWarmupBytes();
								warmupTime = invidualTests[i].getWarmupTime() > warmupTime ? invidualTests[i].getWarmupTime() : warmupTime;
							}
							warmupBytes = bytes;
							startWarmup = getFastStartWarmup();
							
							if(isFinishAllWarmup()){
								readyTransfer = microTime();
								for (int i = 0; i < nThreads; i++) {
									invidualTests[i].setTransferReady(readyTransfer);
								}
								warmUpDone = true;
								Log.e(TAG,"Total warmupBytes:"+warmupBytes);
							}
						}
						else {
							int bytes = 0;
							for (int i = 0; i < nThreads; i++) {
								bytes += invidualTests[i].getTransferBytes();
								transferTime = invidualTests[i].getTransferTime() > transferTime ? invidualTests[i].getTransferTime() : transferTime;
							}
							transferBytes = bytes;
							if((microTime()-readyTransfer) >1000000){
								transferBytesAfter1Sec = transferBytes - transferBytes1SecOffset;
								transferTimeAfter1Sec = microTime() - startTransferAfter1Sec;
							}
							else {
								transferBytes1SecOffset = transferBytes;
								startTransferAfter1Sec = microTime();
							}
						}
						
						//error check
						for (int i = 0; i < nThreads; i++) if(invidualTests[i].error.get()) error.set(true);
						if(error.get()){
							for (int i = 0; i < nThreads; i++) invidualTests[i].error.set(true);
						}
					}

					@Override
					public boolean isWaitForDisconnect(HttpSingleTest test) {
						boolean allTestDone = true;
						for (int i = 0; i < nThreads; i++) {
							if(invidualTests[i].testDone == false){
								allTestDone = false;
								break;
							}
						}
						if(allTestDone){
							try {
								Thread.sleep(100);
							} catch (InterruptedException e) {

							}
							
							for (int i = 0; i < nThreads; i++) {
								synchronized(invidualTests[i]){
									invidualTests[i].notifyAll();
								}
							}	
							return false;
						}
						return true;
					}
				});
			}
			
			for (int i = 0; i < nThreads; i++) {
				if(invidualTests[i].isReady()){
					threads[i] = new Thread(invidualTests[i]);
					threads[i].start();
					threads[i].setName("HTTPTest");
					nCountActiveThread++;	
				}				
			}
		}
		else {
			for (int i = 0; i < nThreads; i++) {
				threads[i] = new Thread(this);
				threads[i].start();
				nCountActiveThread++;
			}
		}
		try {
			for (int i = 0; i < nThreads; i++) {
				threads[i].join();
				nCountActiveThread--;
			}
		} catch (Exception e) {
			setErrorIfEmpty("Thread join exception: ", e);
			SKLogger.sAssert(getClass(),  false);
			testStatus = "FAIL";
		}

		
		isActiveTest = false;
		Log.e(TAG,"HTTP Test Finish###########");

		if (downstream) {
			infoString = HTTPGETDONE;
		} else {
			infoString = HTTPPOSTDONE;
		}

		output();
		finish();
	}

	public String getInfo() {
		return infoString;
	}

	public int getCurrentSpeed() {
		return currentSpeed;
	}
	
	public int getPeakSpeed() {
		return peakSpeed;
	}
	
	// Bytes per second
	public int getSpeed() {		// bytes per sec
		if(transferTime == 0) return 0;
		if(transferBytes == 0) return 0;
		
		return (int) (((transferBytes) / ((double) transferTime) * 1000000));
	}
	
	public int getSpeed2() {
		if(transferTimeAfter1Sec == 0) return 0;
		if(transferBytesAfter1Sec == 0) return 0;
		if(peakSpeed == 0) return 0;
		
		return (int) (((transferBytesAfter1Sec) / ((double) transferTimeAfter1Sec) * 1000000));
	}
	
	private int getSpeed(int bytes, long time) {		// bytes per sec
		if(time == 0) return 0;
		return (int) (((bytes) / ((double) time) * 1000000));
	}

	public synchronized int getTransferBytes() {
		return warmupBytes + transferBytes;
	}

	public synchronized int getWarmupBytes() {
		return warmupBytes;
	}

	/* index : value
	 * 0 : Test.JSON_TYPE
	 * 1 : Test.JSON_TIMESTAMP (UNIX)
	 * 2 : Test.JSON_SUCCESS (FAIL,OK)
	 * 3 : Test.JSON_TARGET (name)
	 * 4 : Test.JSON_TARGET_IPADDRESS (address)
	 * 5 : Test.JSON_TRANFERTIME
	 * 6 : Test.JSON_TRANFERBYTES
	 * 7 : Test.JSON_BYTES_SEC
	 * 8 : Test.JSON_WARMUPTIME
	 * 9 : Test.JSON_WARMUPBYTES
	 * 10 : Test.JSON_NUMBER_OF_THREADS
	 * 11 : Test.JSON_PEAK
	 */
	private void output() {
		ArrayList<String> o = new ArrayList<String>();
		Map<String, Object> output = new HashMap<String, Object>();
		// string id
		o.add(getStringID());
		output.put(Test.JSON_TYPE, getStringID());
		// time
		long time_stamp = unixTimeStamp();
		o.add(time_stamp + "");
		output.put(Test.JSON_TIMESTAMP, time_stamp);
		output.put(Test.JSON_DATETIME, SKDateFormat.sGetDateAsIso8601String(new java.util.Date(time_stamp*1000)));
		// status
		if (error.get()) {
			o.add("FAIL");
			output.put(Test.JSON_SUCCESS, false);
		} else {
			o.add("OK");
			output.put(Test.JSON_SUCCESS, true);
		}
		// target
		o.add(target);
		output.put(Test.JSON_TARGET, target);
		// target ip address
		o.add(ipAddress);
		output.put(Test.JSON_TARGET_IPADDRESS, ipAddress);
		// transfer time
		o.add(Long.toString(transferTime));
		output.put(JSON_TRANFERTIME, transferTime);
		// transfer bytes
		o.add(Integer.toString(transferBytes));
		output.put(JSON_TRANFERBYTES, transferBytes);
		// byets_sec
		o.add(Integer.toString(getSpeed()));
		output.put(JSON_BYTES_SEC, getSpeed());
		// warmup time
		o.add(Long.toString(warmupTime));
		output.put(JSON_WARMUPTIME, warmupTime);
		// warmup bytes
		o.add(Integer.toString(warmupBytes));
		output.put(JSON_WARMUPBYTES, warmupBytes);
		// number of threads
		o.add(Integer.toString(nThreads));
		output.put(JSON_NUMBER_OF_THREADS, nThreads);
		// peak of speed
		o.add(Integer.toString(peakSpeed));	// 11
		output.put(JSON_PEAK_BYTES_SEC, peakSpeed);
		// byte_sec after1sec
		o.add(Integer.toString(getSpeed2()));
		output.put(JSON_BYTES_SEC_AFTER_1SEC, getSpeed2());

		setOutput(o.toArray(new String[1]));
		setJSONOutput(output);
		
		Log.e(TAG,"getSpeed:"+changeMbps(getSpeed())+" getSpeed2:"+changeMbps(getSpeed2()));
	}
	
	private float changeMbps(int bytePerSec){
		bytePerSec *= 8;
		return (bytePerSec/(float)1000000);
	}

	public void download() {


		OutputStream connOut = null;
		InputStream connIn = null;
		byte[] buff = new byte[bufferSize];
		int readBytes = 0;
		HttpURLConnection con = null;
		String urlString = "http://"+target+":"+port+"/"+"104857600";	//100MByte
		
		
		try {
			URL url = new URL(urlString);
			con = (HttpURLConnection) url.openConnection();
			con.setInstanceFollowRedirects(true);
			con.setRequestMethod("GET");
			con.setUseCaches(false);
			con.setDoOutput(false);	
			con.setDoInput(true);
			con.setConnectTimeout(5000);
			
			con.connect();
			connIn = con.getInputStream();
			int httpResponse = con.getResponseCode();
			if (httpResponse != HttpURLConnection.HTTP_OK) {
				setErrorIfEmpty("Http response received: " + httpResponse);
				error.set(true);
				Log.e(TAG, "Http response received: " + httpResponse);
			}
		} catch (MalformedURLException e) {
			error.set(true);
			Log.e(TAG,"MalformedURLException");
		} catch (IOException e) {
			error.set(true);
			Log.e(TAG,"IOException");
		}
		
		waitForAllConnections();
		if (error.get()) {
			closeConnection(con, connIn, connOut);
			return;
		}
		// warmup, can be based on time constraint or data usage constraint
		do {
			try {
				readBytes = connIn.read(buff, 0, buff.length);
			} catch (IOException io) {
				readBytes = BYTESREADERR;
				error.set(true);
			}
		} while (!isWarmupDone(readBytes));
		if (error.get()) {
			closeConnection(con, connIn, connOut);
			return;
		}
		do {
			try {
				readBytes = connIn.read(buff, 0, buff.length);
			} catch (IOException io) {
				readBytes = BYTESREADERR;
			}
		} while (!isTransferDone(readBytes));

		closeConnection(con, connIn, connOut);

	}

	synchronized void waitForAllConnections() {
		connectionCounter++;
		if (connectionCounter < nThreads) {
			try {
				wait();
			} catch (InterruptedException e) {
				error.set(true);
			}
		} else {
			notifyAll();
		}
	}

	// the warmup ends also if there is a problem on any connection
	synchronized boolean isWarmupDone(int bytes) {
		boolean ret = false;
		boolean timeWarmup = false;
		boolean bytesWarmup = false;
		// if there is an error the test must stop and report it
		if (bytes == BYTESREADERR) {
			setErrorIfEmpty("read error");
			bytes = 0; // do not modify the bytes counters
			error.set(true);
		}

		warmupBytes += bytes;
		if (startWarmup == 0) {
			startWarmup = microTime();
		}
		warmupTime = microTime() - startWarmup;
		
		// if warmup max time is set and time has exceeded its values set time
		// warmup to true
		timeWarmup = warmupMaxTime > 0 && warmupTime >= warmupMaxTime;

		// if warmup max bytes is set and bytes counter exceeded its value set
		// bytesWarmup to true
		bytesWarmup = warmupMaxBytes > 0 && warmupBytes >= warmupMaxBytes;

		// if a condition happened increment the warmupDoneCounter
		if (timeWarmup || bytesWarmup) {
			warmupDoneCounter++;
			ret = true;
		}
		// if there is an error notify all, some thread might be waiting for
		// other to finish the warmup, and exit
		if (error.get()) {
			notifyAll();
			ret = true;

		}// warmup is finished, wait for other threads if any
		else if (ret && warmupDoneCounter < nThreads) {
			//startTransfer = microTime();
			readyTransfer = microTime();
			try {
				wait();
			} catch (InterruptedException ie) {
				error.set(true);
				ret = true;
				notifyAll();
			}
		}// warmup is finished, last thread, notify all
		else if (ret) {
			warmUpDone = true;
			notifyAll();
		}
		return ret;

	}

	synchronized boolean isTransferDone(int bytes) {
		boolean ret = false;
		
		// during 1sec discard
		if(readyTransfer > 0){
			long readyTime = microTime() - readyTransfer;
			if(readyTime < 1000000){
				return false;
			}
			readyTransfer = 0;
			startTransfer = microTime();
			Log.d(TAG,"readyTime"+readyTime+" real transfer start!");
			
			//initial
			currentSpeed = 0;
			peakSpeed = 0;
			return false;
		}
		
		// In case an error occurred stop the test
		if (bytes == BYTESREADERR) {
			error.set(true);
			bytes = 0;
		}
		transferBytes += bytes;

		// if startTransfer is 0 this is the first call to isTransferDone
		if (startTransfer == 0) {
			startTransfer = microTime();
		}
		transferTime = microTime() - startTransfer;
		
		// if transfermax time is
		if ((transferMaxTime > 0) && (transferTime > transferMaxTime)) {
			ret = true;
		}
		if ((transferMaxBytes > 0)
				&& (transferBytes + warmupBytes > transferMaxBytes)) {
			ret = true;
		}
		if (transferBytes > 0) {
			testStatus = "OK";
		}

		if (error.get()) {
			ret = true;
		}

		return ret;
	}

	private int nSessionCountDownload;
	private int nSessionCountUpload;
	@Override
	public void run() {

		if (downstream) {
			download();
		} else {
			upload();
		}
	}

	public void upload() {

		
		OutputStream connOut = null;
		InputStream connIn = null;
		Random generator = new Random();
		byte[] buff = new byte[sendDataChunkSize];
		HttpURLConnection con = null;
		String urlString = "http://"+target+":"+port;

		try {
			URL url = new URL(urlString);
			con = (HttpURLConnection) url.openConnection();
			con.setRequestMethod("POST");
			con.setUseCaches(false);
			con.setDoOutput(true);
			con.setChunkedStreamingMode(0);
//			con.setRequestProperty("Content-Length",""+postDataLength);
			con.setRequestProperty("Content-Type","application/x-www-form-urlencoded");
			con.setRequestProperty("Expect","100-continue");
			con.setConnectTimeout(5000);
		
			con.connect();
			connOut = con.getOutputStream();
		} catch (MalformedURLException e) {
			error.set(true);
		} catch (IOException e) {
			error.set(true);
		}
		
		waitForAllConnections();
		if (error.get()) {
			closeConnection(con, connIn, connOut);
			return;
		}

		do {
			if (randomEnabled) {
				generator.nextBytes(buff);
			}
			try {
				connOut.write(buff);
				connOut.flush();
			} catch (IOException ioe) {
				error.set(true);
			}
		} while (!isWarmupDone(sendDataChunkSize));
		if (error.get()) {
			closeConnection(con, connIn, connOut);
			return;
		}
		do {
			if (randomEnabled) {
				generator.nextBytes(buff);
			}
			try {
				connOut.write(buff);
				connOut.flush();
			} catch (IOException ioe) {
				error.set(true);
			}
		} while (!isTransferDone(sendDataChunkSize));
		if (error.get()) {
			closeConnection(con, connIn, connOut);
			return;
		}
		
		try {
			int httpResponse = con.getResponseCode();
			if (httpResponse != HttpURLConnection.HTTP_OK) {
				setErrorIfEmpty("Http response received: " + httpResponse);
				error.set(true);
			}
		} catch (IOException ioe) {
			error.set(true);
		}		
		closeConnection(con, connIn, connOut);
		Log.i(TAG,"upload finish~~~~!!!");
	}

	private void closeConnection(HttpURLConnection h, InputStream i, OutputStream o) {
		try {
			if(i != null){
				i.close();
			}
			if(o != null){
				o.close();
			}
			if(h != null){
				h.disconnect();
			}
		} catch (IOException ioe) {

		}
	}
	

	public void setTarget(String target) {
		this.target = target;
	}

	public void setPort(int port) {
		this.port = port;
	}

	public void setFile(String file) {
		this.file = file;
	}

	public void setWarmupMaxTime(int time) {
		warmupMaxTime = time;
	}

	public void setWarmupMaxBytes(int bytes) {
		warmupMaxBytes = bytes;
	}

	public void setTransferMaxTime(int time) {
		transferMaxTime = time;
	}

	public void setTransferMaxBytes(int bytes) {
		transferMaxBytes = bytes;
	}

	public void setBufferSize(int size) {
		bufferSize = size;
	}

	public void setSendBufferSize(int size) {
		desideredSendBufferSize = size;
	}

	public void setReceiveBufferSize(int size) {
		desideredReceiveBufferSize = size;
		bufferSize = size;
	}

	public void setSendDataChunk(int size) {
		sendDataChunkSize = size;
	}

	public void setPostDataLenght(int l) {
		postDataLength = l;
	}

	public void setDownstream() {
		downstream = true;
	}

	public void setUpstream() {
		downstream = false;
	}

	public void setNumberOfThreads(int n) {
		nThreads = n;
	}

	public void setDirection(String d) {
		if (Test.paramMatch(d, DOWNSTREAM)) {
			downstream = true;
		} else if (Test.paramMatch(d, UPSTREAM)) {
			downstream = false;
		}
	}

	public boolean isProgressAvailable() {
		boolean ret = false;
		if (transferMaxTime > 0) {
			ret = true;
		} else if (transferMaxBytes > 0) {
			ret = true;
		}
		return ret;
	}

	public int getProgress() {
		double ret = 0;

		synchronized (this) {
			if (startWarmup == 0) {
				ret = 0;
			} else if (transferMaxTime != 0) {
				long currTime = microTime() - startWarmup;
				ret = (double) currTime / (warmupMaxTime + transferMaxTime);

			} else {
				int currBytes = warmupBytes + transferBytes;
				ret = (double) currBytes / (warmupMaxBytes + transferMaxBytes);
			}
		}
		ret = ret < 0 ? 0 : ret;
		ret = ret >= 1 ? 0.99 : ret;
		return (int) (ret * 100);
	}

	boolean upload = true;
	boolean randomEnabled = false;
	String infoString = "";
	String ipAddress = "";
	boolean warmUpDone = false;
	int postDataLength = 0;

	// warmup variables
	long startWarmup = 0;
	long warmupTime = 0;
	long warmupMaxTime = 0;
	int warmupBytes = 0;
	int warmupMaxBytes = 0;
	int warmupDoneCounter = 0;

	// transfer variables
	long readyTransfer = 0;
	long startTransfer = 0;
	long startTransferAfter1Sec = 0;
	long transferTime = 0;
	long transferTimeAfter1Sec = 0;
	int transferBytes = 0;
	int transferBytes1SecOffset = 0;
	int transferBytesAfter1Sec = 0;
	long transferMaxTime = 0;
	int transferMaxBytes = 0;
	int transferDoneCounter = 0;
	int peakSpeed = 0;
	int currentSpeed = 0;


	// test variables
	int connectionCounter = 0;
	int nThreads;
	boolean noDelay = false;
	int receiveBufferSize = 0;
	int sendBufferSize = 0;
	int bufferSize = 0;
	int desideredReceiveBufferSize = 0;
	int desideredSendBufferSize = 0;
	int sendDataChunkSize = 0;
	String testStatus = "FAIL";

	// Connection variables
	String target = "";
	String file = "";
	int port = 0;
	boolean downstream = true;

	// Test Status variable
	private AtomicBoolean error = new AtomicBoolean(false);

}
