package com.accuver.cem.view.frag;


import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.graphics.Typeface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.accuver.cem.struc.HistoryItem;
import com.accuver.cem.view.util.HistoryDetailDialog;
import com.accuver.cem.view.util.TypeFaces;
import com.accuver.jpcem.R;
import com.samknows.measurement.storage.ExportFile;

public class Frag_History  extends Fragment{

	private static final String TAG = Frag_History.class.getSimpleName();
	private ListView lv_history;
	private HistoryListAdapter as_adapter;
	private List<HistoryItem> tech_items; //for listview HistoryItem (reversed)
	
	private int mTechSelected = 0, mTmpSelected = 0;
	private HistoryDetailDialog hdd = null;
	private Activity mActivity = null;

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		Log.d(TAG, "onCreateView()");
		mActivity = getActivity();
		View rootView = inflater.inflate(R.layout.frag_history, container, false);
		findView(rootView);
		return rootView;
	}

	@Override
	public void onResume() {
		super.onResume();
		Log.d(TAG, "onResume()");
		readFileThread();
	}
	
	@Override
	public void onPause() {
		super.onPause();
		Log.d(TAG, "onPause()");
	}
	
	@Override
	public void onStart() {
		super.onStart();
		Log.d(TAG, "onStart()");
	}
	
	private void findView(View view) {
		
		TextView tv_0 = (TextView) view.findViewById(R.id.tv_0);
		tv_0.setTypeface(TypeFaces.get(mActivity, "NeoSans-Medium.otf"), Typeface.BOLD);
		
		// Top Layer to do some actions
		((TextView) view.findViewById(R.id.btn_tech_select)).setOnClickListener(mClickListener);
		((TextView) view.findViewById(R.id.btn_history_clear)).setOnClickListener(mClickListener);

		lv_history = (ListView) view.findViewById(R.id.lv_history);
		as_adapter = new HistoryListAdapter(mActivity);
		lv_history.setAdapter(as_adapter);
		lv_history.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> paramAdapterView, View paramView, int paramInt, long paramLong) {
				if(hdd == null) {
					hdd = new HistoryDetailDialog(mActivity, tech_items.get(paramInt)); 
				}else{
					hdd.setItem(tech_items.get(paramInt)); 
				}
				hdd.setCanceledOnTouchOutside(false);
				hdd.setCancelable(false);
				hdd.show();

				Log.e(TAG,"click item");
			}
		});
		//readFileThread();
	}

	private Thread mRefreshThread = null;
	public void readFileThread() {
		if(mRefreshThread != null){
			if(mRefreshThread.isAlive()){
				return;
			}
		}
		mRefreshThread = new Thread(new Runnable() {
			@Override
			public void run() {
				String[] lines = ExportFile.readLineFromCSV("history.csv");
				List<HistoryItem> items = new ArrayList<HistoryItem>();
				if(lines == null){
					tech_items = items;
					mActivityHandler.post(new Runnable(){
						@Override
						public void run() {
							if(lv_history != null)lv_history.invalidateViews();
						}
					});
					return;
				}
				
				try{
					for(int i=lines.length-1; i>0; i--){
						String line = lines[i];
						String[] info = line.split(",");
						if(info.length != ExportFile.HISTORY_FIELDS.length){
							throw new Exception("wrong history type");
						}
						
						HistoryItem item = new HistoryItem();
						item.mNo = i;
						item.Start_Date = info[0].replace(ExportFile.FIELD_DELIMITER, "");
						item.End_Date = info[1].replace(ExportFile.FIELD_DELIMITER, "");
						item.mNet = info[2].replace(ExportFile.FIELD_DELIMITER, "");
						item.Cell_ID = info[3].replace(ExportFile.FIELD_DELIMITER, "");
						item.PCI = info[4].replace(ExportFile.FIELD_DELIMITER, "");
						item.SINR = info[5].replace(ExportFile.FIELD_DELIMITER, "");
						item.RSRP = info[6].replace(ExportFile.FIELD_DELIMITER, "");
						item.RSRQ = info[7].replace(ExportFile.FIELD_DELIMITER, "");
						item.RSSI = info[8].replace(ExportFile.FIELD_DELIMITER, "");
						item.Latency = info[9].replace(ExportFile.FIELD_DELIMITER, "");
						item.PacketLoss = info[10].replace(ExportFile.FIELD_DELIMITER, "");
						item.Max_DL_Throughput = info[11].replace(ExportFile.FIELD_DELIMITER, "");
						item.Max_UL_Throughput = info[12].replace(ExportFile.FIELD_DELIMITER, "");
						item.Avg_DL_Throughput = info[13].replace(ExportFile.FIELD_DELIMITER, "");
						item.Avg_UL_Throughput = info[14].replace(ExportFile.FIELD_DELIMITER, "");
						item.Gps_latitude = info[15].replace(ExportFile.FIELD_DELIMITER, "");
						item.Gps_Longitude = info[16].replace(ExportFile.FIELD_DELIMITER, "");
						item.Gps_Accuracy = info[17].replace(ExportFile.FIELD_DELIMITER, "");

						items.add(item);
					}
					tech_items = items;
					mActivityHandler.post(new Runnable(){
						@Override
						public void run() {
							Log.d(TAG,"list update");
							if(lv_history != null)lv_history.invalidateViews();
						}
					});

				}
				catch(Exception e){
					Log.e(TAG,""+e.getMessage());
				}
			}
		});
		mRefreshThread.start();
	}

	public Handler mActivityHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			
		}
	};

	View.OnClickListener mClickListener = new View.OnClickListener() {
		@Override
		public void onClick(View view) {
			Log.e(TAG,"[mClickListener] : click func");
			int id = view.getId();
			switch (id) {
			case R.id.btn_history_clear: {
				ExportFile.deleteFile("history.csv");
				readFileThread();
				break;
			}
			}
		}
	};

//	private static final String items[] = { "ALL", "LTE", "3G", "2G", "Wifi", "N/A" };
//	private void techSelectDialog() {
//		AlertDialog.Builder ab = new AlertDialog.Builder(mActivity);
//		ab.setTitle("Select network tech");
//		ab.setSingleChoiceItems(items, mTechSelected, new DialogInterface.OnClickListener() {
//			public void onClick(DialogInterface dialog, int whichButton) {
//				mTmpSelected = whichButton;
//			}
//		}).setPositiveButton("Ok", new DialogInterface.OnClickListener() {
//			public void onClick(DialogInterface dialog, int whichButton) {
//				mTechSelected = mTmpSelected;
//				Log.e(TAG,"[techSelectDialog] : ok = "+mTechSelected);
//				as_adapter.setTechDefined(mTechSelected);
//				as_adapter.notifyDataSetChanged();
//				as_adapter.notifyDataSetInvalidated();
//				((TextView) mActivity.findViewById(R.id.btn_tech_select)).setText(String.valueOf(items[mTechSelected]));
//			}
//		}).setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
//			public void onClick(DialogInterface dialog, int whichButton) {
//				
//			}
//		});
//		ab.show();
//	}

	public class HistoryListAdapter extends BaseAdapter {

		public static final int MAX_ITEM = 100;

		private LayoutInflater mInflater;

		public HistoryListAdapter(Context context){
			mInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		}
		
		@Override
		public View getView(final int position, View convertView, ViewGroup parent) {
			if (convertView == null) {
				convertView = mInflater.inflate(R.layout.list_history_item, parent, false);
			}
			TextView tv_no = (TextView) convertView.findViewById(R.id.tv_no);
			TextView tv_network = (TextView) convertView.findViewById(R.id.tv_network);

			TextView tv_val_3 = (TextView) convertView.findViewById(R.id.tv_val_3);
			TextView tv_val_4 = (TextView) convertView.findViewById(R.id.tv_val_4);
			TextView tv_val_5 = (TextView) convertView.findViewById(R.id.tv_val_5);
			
			TextView tv_loss = (TextView) convertView.findViewById(R.id.tv_packet_loss);
			TextView tv_down_max = (TextView) convertView.findViewById(R.id.tv_down_max);
			TextView tv_up_max = (TextView) convertView.findViewById(R.id.tv_up_max);
			
			
			
			if (tech_items != null && tech_items.size() > position) {
				try {
					final HistoryItem item = tech_items.get(position);
					
					String End_Time = "";
					String[] timetemp = item.End_Date.split("_");

					if(timetemp.length > 0)
						End_Time = timetemp[1].substring(0, 2)+":"+timetemp[1].substring(2, 4)+"\n"+timetemp[0].substring(2, 4)+"/"+timetemp[0].substring(4, 6)+"/"+timetemp[0].substring(6, 8);
					else
						End_Time = "N/A";

					tv_no.setText(End_Time);

					if(item.mNet.equals("WIFI"))
					{
						tv_network.setBackgroundResource(R.drawable.cem_history_wifi_48);
						tv_network.setText("");
					}
					else
					{
						tv_network.setBackgroundColor(Color.parseColor("#ffa500"));
						tv_network.setText(String.valueOf(item.mNet));
					}

					tv_val_3.setText(getLatency(item.Latency));
					tv_val_4.setText(getThroughput(item.Avg_DL_Throughput));
					tv_val_5.setText(getThroughput(item.Avg_UL_Throughput));
					tv_loss.setText("(L:"+getLoss(item.PacketLoss)+"%)");
					tv_down_max.setText("(P:"+getThroughput(item.Max_DL_Throughput)+")"); 
					tv_up_max.setText("(P:"+getThroughput(item.Max_UL_Throughput)+")");
					
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			return convertView;
		}
		
		private String getLoss(String value){
			if(value.equalsIgnoreCase("null")){
				return "N/A";
			}
			double value_double = Double.parseDouble(value);
			value = String.format(Locale.getDefault(), "%.1f", value_double);
			return value;
		}
		
		private String getLatency(String value){
			if(value.equalsIgnoreCase("null")){
				return "N/A";
			}
			double value_double = Double.parseDouble(value);
			value_double /= 1000;
			value = String.format(Locale.getDefault(), "%.1f", value_double);
			return value;
		}
		private String getThroughput(String value){
			if(value.equalsIgnoreCase("null")){
				return "N/A";
			}
			double value_double = Double.parseDouble(value);
			value_double /= 1000000;
			value = String.format(Locale.getDefault(), "%.1f", value_double);
			return value;
		}
		

		@Override
		public int getCount() {
			if (tech_items != null)
				return tech_items.size();
			return 0;
		}

		@Override
		public Object getItem(int position) {
			return null;
		}

		@Override
		public long getItemId(int position) {
			return 0;
		}
		

	}
}
